Package: posDemux
Type: Package
Title: Positional combinatorial sequence demultiplexer
Encoding: UTF-8
Version: 1.0.0
Date: 2025-11-05
Authors@R: c(
   person(given = "Jakob Peder",
   family = "Pettersen",
   role = c("aut","cre"),
   email = "jakobpeder.pettersen@gmail.com",
   comment = c(ORCID = "0000-0002-3485-1634")
   ),
   person("Centre for new antibacterial strategies (CANS)",
   role = "fnd"
   )
   )
Description: Demultiplexing and filtering utilities intended for reads
        with combinatorial barcodes (i.e. PETRI-seq and SPLiT-seq). The
        demultiplexer algorithm uses the position of the segments to
        extract and compare the barcodes with the reference
        (whitelist). A Shiny application is provided to interactively
        select cutoffs for which barcode combinations to keep.
License: AGPL (>= 3)
Depends: R (>= 4.5.0)
Imports: Biostrings, ggplot2, methods, assertthat, glue, magrittr,
        dplyr, rlang, ShortRead, readr, shiny, purrr
LinkingTo: Rcpp, Biostrings, IRanges, S4Vectors, XVector
biocViews: SequenceMatching, Sequencing, Software, RNASeq
URL: https://github.com/yaccos/posDemux,
        https://yaccos.github.io/posDemux/
BugReports: https://github.com/yaccos/posDemux/issues
RoxygenNote: 7.3.3
Roxygen: list(markdown = TRUE)
Suggests: testthat, devtools, DNABarcodes, knitr, rmarkdown, tibble,
        tidyr, BiocStyle, RefManageR, sessioninfo, DBI, chunked,
        RSQLite, dbplyr
Config/testthat/edition: 3
VignetteBuilder: knitr
Config/pak/sysreqs: cmake make libbz2-dev libjpeg-dev liblzma-dev
        libpng-dev libuv1-dev libx11-dev xz-utils zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 13:06:04 UTC
RemoteUrl: https://github.com/bioc/posDemux
RemoteRef: RELEASE_3_23
RemoteSha: 053127fe948081a3aa5a62a601ef30e1a9537f09
NeedsCompilation: yes
Packaged: 2026-04-29 23:55:11 UTC; root
Author: Jakob Peder Pettersen [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3485-1634>),
  Centre for new antibacterial strategies (CANS) [fnd]
Maintainer: Jakob Peder Pettersen <jakobpeder.pettersen@gmail.com>
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-29 23:58:53 UTC; unix
