# Generated by roxygen2: do not edit by hand

S3method(print,demultiplex_filter_summary)
export(bc_to_freq_cutoff)
export(combinatorial_demultiplex)
export(create_freq_table)
export(create_summary_res)
export(filter_demultiplex_res)
export(freq_plot)
export(freq_to_bc_cutoff)
export(interactive_bc_cutoff)
export(knee_plot)
export(row_match)
export(streaming_callbacks)
export(streaming_demultiplex)
import(assertthat)
import(dplyr)
import(ggplot2)
import(glue)
import(shiny)
importFrom(Biostrings,subseq)
importFrom(Biostrings,width)
importFrom(Biostrings,xscat)
importFrom(magrittr,"%<>%")
importFrom(magrittr,"%>%")
importFrom(magrittr,add)
importFrom(magrittr,equals)
importFrom(magrittr,extract)
importFrom(magrittr,set_names)
importFrom(magrittr,subtract)
importFrom(methods,is)
importFrom(purrr,imap)
importFrom(purrr,imap_int)
importFrom(purrr,iwalk)
importFrom(purrr,list_cbind)
importFrom(purrr,map)
importFrom(purrr,map2)
importFrom(purrr,map_int)
importFrom(purrr,pmap)
importFrom(purrr,walk2)
importFrom(rlang,.data)
importFrom(utils,combn)
importFrom(utils,head)
useDynLib(posDemux, .registration = TRUE)
