
# posDemux

<!-- badges: start -->
<!-- badges: end -->

The goal of posDemux is to provide tools for tools for handling reads with
combinatorial barcodes, and multiple adapter regions.
This includes utilities for demultiplexing,
sequence segmenting and filtering.
The package is intended to work with single-cell RNA-seq data
with multiple barcoding (e.g. SPLiT-seq and PETRI-seq).

## Installation

You can install the development version of posDemux from [GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("yaccos/posDemux")
```

## Usage

Please see the package [vignettes](https://yaccos.github.io/posDemux/). 

## Issues

Please use the repository's issue tracker if you cannot make the package work or
if you have suggestions for improvements.